%% -*- octave -*-
%%
%% this is the main function that computes the spectral gap 
%% from the impulse response 
%%
%% A : transition matrix 
%% k : length of the impulse response to use
%%
%% G : matrix where column j represents an estimate at time j\in
%% {1,3,5,...,\floor{k/2}*2+1.} Each element i in a column represents
%% the estimate of node i.
%%
function G = impulse_gap_full_adaptive(A, T=0.000001, L=5)
  n = size(A)(1);
  T = T*n*L;
  A = make_col_stochastic(A);
  b = init_b(n);
  G = [];
  H = [ b ];
  for j = 1:n
    b = A*b;
    H = [ H b ];
    b = A*b;
    H = [ H b ];
    Gj = [];
    for i = 1:n
      Gj = [ Gj ; gap_estimate(H(i,:)) ];
    endfor
    if (j > L)
      d = 0;
      for k = (j-L):(j-1)
	d = d + norm(Gj - G(:,k));
      endfor
%%      printf("k=%d d=%f\n", k, d);
      if (d < T)
	return;
      endif
    endif
    G = [ G Gj ];
  endfor
%%  printf("\n");
endfunction
