%% -*- octave -*-
%%
%% this is the main function that computes the spectral gap from the
%% impulse response.
%%
%% A : transition matrix
%%
%% k : minimum length of the impulse response to use rounded up to the
%% nearest odd number
%%
%% G : vector containing the estimates of the spectral gap for each
%% vertex
%%
function k = impulse_gap_incremental(A, k)
  k = floor(k/2)*2+1;
  n = size(A)(1);
  A = make_col_stochastic(A);
  b = init_b(n);
  H = [ b ];
  for i = (2:k)
    b = A*b;
    H = [ H b ];
  endfor
  Gp = [];
  for i = 1:n
    Gp = [ Gp ; gap_estimate(H(i,:)) ];
  endfor
  %% we skip by two because Kung's algorithm would build and use exactly
  %% the same Henkel matrix for impulse responses of lengths k and k+1
  b = A*b;
  H = [ H b ];
  b = A*b;
  H = [ H b ];
  %%
  G = [];
  for i = 1:n
    G = [ G ; gap_estimate(H(i,:)) ];
  endfor
  while (Gp != G)
    Gp = G;
    k = k + 2;
    b = A*b;
    H = [ H b ];
    b = A*b;
    H = [ H b ];
    %%
    G = [];
    for i = 1:n
      G = [ G ; gap_estimate(H(i,:)) ];
    endfor
  endwhile
  G
