%% this is the main function that computes the spectral gap from the
%% impulse response.
%%
%% A : transition matrix
%%
%% k : minimum length of the impulse response to use rounded up to the
%% nearest odd number
%%
%% G : vector containing the estimates of the spectral gap for each
%% vertex
%%
function d = impulse_gap_k_diff(A,k,max_k=201)
  
  n = size(A)(1);
  A = make_col_stochastic(A);
  b = init_b_nonzero(n);
  H = impulse_response(A,b,max_k);
  Gmax = [];
  for i = 1:n
    Gmax = [ Gmax ; gap_estimate_nonzero_b(H(i,:)) ];
  endfor
  G = [];
  for i = 1:n
    G = [ G ; gap_estimate_nonzero_b(H(i,1:k)) ];
  endfor
  D = G - Gmax;
  N = norm(D);
  SD = sum(abs(D))/size(D)(1)
  d = max(abs(G - Gmax));
  printf("SD = %f, N = %f, d = %f\n", SD, N, d);
  