% -*- octave -*-
% 
% Kung's algorithm
% 
function A = kung_with_rank(h)
  k = size(h)(2);
  %% we assume h is a 1xk matrix -- i.e., a row vector of size k

  %% k must be odd; if not we cut h to its max odd size
  l = floor((k+1)/2);
  k = l*2 - 1;

  %% then we build the Henkel matrix out of h
  %% H = [];
  %% for i = 1:l
  %% H = [ H ; h(i:(i+l-1)) ];
  %% end
  H = hankel(h(1:l),h(l:k));
  %%
  %% then we compute the SVD of H
  [U,S,V] = svd(H,0);
  s = diag(S);
  %TOL = 1e-7;
  %n = sum(s > TOL);
  n=rank(H);

  S_sqrt = sqrt(s(1:n))';
  O =  U(:,1:n) * diag(S_sqrt);
  %% the observability matrix satisfies the following equation:
  %% O_l * A = O_r
  %% where O_l is the matrix of the first n-1 rows of the observability
  %% matrix, and O_r is the matrix of the last n-1 rows of the
  %% observability matrix.
  %%
  %% So, we obtain A by right-multiplying O_r by O_l's pseudoinverse
  %% W = X \ Y means W = Y multiplied by X's pseudo-inverse
  %%
  A = O(1:end-1,:)\O(2:end,:);
  if any(any(isnan(A)))
    printf("Warning: NaN results, try to apply the pseudoinverse function\n");
    A = pinv(O(1:end-1,:))*O(2:end,:);
    %%error check
    error=abs(O(1:end-1,:)*A - O(2:end,:));
    if(max(max(error))>1e-7)
      printf("Error: A is not an exact solution\n");
    endif
 endif
endfunction
